@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 2006-2009 Bjorn Andersson <flex@kryo.se>, Erik Ekman <yarrick@kryo.se>
+ * Copyright (c) 2006-2014 Erik Ekman <yarrick@kryo.se>,
+ * 2006-2009 Bjorn Andersson <flex@kryo.se>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -50,6 +51,7 @@ START_TEST(test_base32_encode)
 	len = sizeof(buf);
 	val = b32->encode(buf, &len, testpairs[_i].a, strlen(testpairs[_i].a));
 
+	fail_unless(val == strlen(testpairs[_i].b));
 	fail_unless(strcmp(buf, testpairs[_i].b) == 0,
 			"'%s' != '%s'", buf, testpairs[_i].b);
 }
@@ -61,13 +63,13 @@ START_TEST(test_base32_decode)
 	char buf[4096];
 	struct encoder *b32;
 	int val;
-	
+
 	b32 = get_base32_encoder();
 
 	len = sizeof(buf);
 	val = b32->decode(buf, &len, testpairs[_i].b, strlen(testpairs[_i].b));
 
-	fail_unless(buf != NULL, "buf == NULL");
+	fail_unless(val == strlen(testpairs[_i].a));
 	fail_unless(strcmp(buf, testpairs[_i].a) == 0,
 			"'%s' != '%s'", buf, testpairs[_i].a);
 }
@@ -79,7 +81,7 @@ START_TEST(test_base32_5to8_8to5)
 	int c;
 
 	for (i = 0; i < 32; i++) {
-		c = b32_5to8(i);	
+		c = b32_5to8(i);
 		fail_unless(b32_8to5(c) == i);
 	}
 }
