@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 2006-2007 Bjorn Andersson <flex@kryo.se>, Erik Ekman <yarrick@kryo.se>
+ * Copyright (c) 2006-2014 Erik Ekman <yarrick@kryo.se>,
+ * 2006-2009 Bjorn Andersson <flex@kryo.se>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -38,6 +39,9 @@ main()
 	test = test_base64_create_tests();
 	suite_add_tcase(iodine, test);
 
+	test = test_common_create_tests();
+	suite_add_tcase(iodine, test);
+
 	test = test_dns_create_tests();
 	suite_add_tcase(iodine, test);
 
@@ -53,8 +57,11 @@ main()
  	test = test_user_create_tests();
 	suite_add_tcase(iodine, test);
 
+ 	test = test_fw_query_create_tests();
+	suite_add_tcase(iodine, test);
+
 	runner = srunner_create(iodine);
-	srunner_run_all(runner, CK_MINIMAL);
+	srunner_run_all(runner, CK_NORMAL);
 	failed = srunner_ntests_failed(runner);
 
 	srunner_free(runner);
