@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 2006-2009 Bjorn Andersson <flex@kryo.se>, Erik Ekman <yarrick@kryo.se>
+ * Copyright (c) 2006-2014 Erik Ekman <yarrick@kryo.se>,
+ * 2006-2009 Bjorn Andersson <flex@kryo.se>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -19,8 +20,8 @@
 #include <string.h>
 #include <time.h>
 #include <sys/socket.h>
-#include <arpa/inet.h>
 #include <netinet/in.h>
+#include <arpa/inet.h>
 
 #include "common.h"
 #include "encoding.h"
@@ -32,10 +33,11 @@ START_TEST(test_init_users)
 	in_addr_t ip;
 	char givenip[16];
 	int i;
+	int count;
 
 	ip = inet_addr("127.0.0.1");
-	init_users(ip, 27);
-	for (i = 0; i < USERS; i++) {
+	count = init_users(ip, 27);
+	for (i = 0; i < count; i++) {
 		fail_unless(users[i].id == i);
 		fail_unless(users[i].q.id == 0);
 		fail_unless(users[i].inpacket.len == 0);
@@ -60,11 +62,12 @@ START_TEST(test_users_waiting)
 	fail_unless(users_waiting_on_reply() == 0);
 
 	users[3].last_pkt = time(NULL);
-	
+
 	fail_unless(users_waiting_on_reply() == 0);
-	
+
+	users[3].conn = CONN_DNS_NULL;
 	users[3].q.id = 1;
-	
+
 	fail_unless(users_waiting_on_reply() == 1);
 }
 END_TEST
@@ -76,20 +79,26 @@ START_TEST(test_find_user_by_ip)
 
 	ip = inet_addr("127.0.0.1");
 	init_users(ip, 27);
+	users[0].conn = CONN_DNS_NULL;
 
 	testip = (unsigned int) inet_addr("10.0.0.1");
 	fail_unless(find_user_by_ip(testip) == -1);
-	
+
 	testip = (unsigned int) inet_addr("127.0.0.2");
 	fail_unless(find_user_by_ip(testip) == -1);
-	
+
 	users[0].active = 1;
-	
+
 	testip = (unsigned int) inet_addr("127.0.0.2");
 	fail_unless(find_user_by_ip(testip) == -1);
-	
+
 	users[0].last_pkt = time(NULL);
-	
+
+	testip = (unsigned int) inet_addr("127.0.0.2");
+	fail_unless(find_user_by_ip(testip) == -1);
+
+	users[0].authenticated = 1;
+
 	testip = (unsigned int) inet_addr("127.0.0.2");
 	fail_unless(find_user_by_ip(testip) == 0);
 }
@@ -103,17 +112,23 @@ START_TEST(test_all_users_waiting_to_send)
 	init_users(ip, 27);
 
 	fail_unless(all_users_waiting_to_send() == 1);
-	
+
+	users[0].conn = CONN_DNS_NULL;
 	users[0].active = 1;
-	
+
 	fail_unless(all_users_waiting_to_send() == 1);
-	
+
 	users[0].last_pkt = time(NULL);
-	
+	users[0].outpacket.len = 0;
+
 	fail_unless(all_users_waiting_to_send() == 0);
 
+#ifdef OUTPACKETQ_LEN
+	users[0].outpacketq_filled = 1;
+#else
 	users[0].outpacket.len = 44;
-	
+#endif
+
 	fail_unless(all_users_waiting_to_send() == 1);
 }
 END_TEST
@@ -127,7 +142,11 @@ START_TEST(test_find_available_user)
 	init_users(ip, 27);
 
 	for (i = 0; i < USERS; i++) {
+		users[i].authenticated = 1;
+		users[i].authenticated_raw = 1;
 		fail_unless(find_available_user() == i);
+		fail_if(users[i].authenticated);
+		fail_if(users[i].authenticated_raw);
 	}
 
 	for (i = 0; i < USERS; i++) {
@@ -140,7 +159,7 @@ START_TEST(test_find_available_user)
 	fail_unless(find_available_user() == -1);
 
 	users[3].last_pkt = 55;
-	
+
 	fail_unless(find_available_user() == 3);
 	fail_unless(find_available_user() == -1);
 }
@@ -168,7 +187,7 @@ START_TEST(test_find_available_user_small_net)
 	fail_unless(find_available_user() == -1);
 
 	users[3].last_pkt = 55;
-	
+
 	fail_unless(find_available_user() == 3);
 	fail_unless(find_available_user() == -1);
 }
